
--============================================================
-- number of objects defined in this file

function lua_getNbObjects()
	return 7
end

--============================================================
-- global variables

description = ''
time_description = 0

last_load_or_save_time = 0

is_game_over = 0

black_sprite = 0

balloon = 0
text_balloon = ''

time_main_music = 0
time_quiet_music = 0

foundTramwayTicket = 0

fall_in_water_sprite = 0

is_player_in_water = false

sound = {splash=0, main_music=0, quiet_music=0}
model = {}

no_key_shoot = 0

is_night = false

--============================================================
--init/deinit

function lua_onInitGame()

	sound.splash = loadSound('sound/splash.wav')
	sound.main_music = loadSound('music/start.wav')
	sound.quiet_music = loadSound('music/feuille.wav')
	
	balloon = load2DSprite('balloon.png')
	black_sprite = load2DSprite('black.png')
	
	fall_in_water_sprite = load2DSprite('menus/fall_in_water.png')
	
	model.delta_wing = load3DModel('obj_delta_wing/delta_wing.obj')
	
	time_main_music = getTime()
	time_quiet_music = getTime() + 120000 --120 seconds
	setPlayerPosition(0, 900, 0)
	callLoadGame()
end

function lua_onUninitGame()
	unload3DModel(model.delta_wing)
	unload2DSprite(black_sprite)
	unload2DSprite(balloon)
	unload2DSprite(fall_in_water_sprite)
end

--============================================================
-- cube end

function lua_load_0()
	model.end_cube = load3DModel('obj_cube_end/cube_end.obj')
end

function lua_manage_0()
	if getDistanceToPlayer() < 15.0 then
		is_game_over = 1
	end
end

function lua_draw_0()
	draw3DModel(model.end_cube)
end

function lua_unload_0()
	unload3DModel(model.end_cube)
end

--============================================================

function lua_load_1()
	model.tree = load3DModel('obj_tree/tree.obj')
end

function lua_manage_1()
	if getDistanceToPlayer() < 10.0 then
		local sx, sy, sz = getPlayerSpeed()
		setPlayerSpeed(-sx, -sy, -sz)
	end
end

function lua_draw_1()
	draw3DModel(model.tree)
end

function lua_unload_1()
	unload3DModel(model.tree)
end

--============================================================

function lua_load_2()
	model.tramway = load3DModel('obj_tramway/tramway.obj')
end

function lua_manage_2()
	if getDistanceToPlayer() < 15 then
		if foundTramwayTicket == 1 then
			x,y,z = getCurrentObjectPosition()
			if x < 1800 then
				x = x + getPeriodDuration() / 15000.0
				setCurrentObjectPosition(x,y,z)
				setPlayerPosition(x,y,z)
				if getPlayerMovementType() == 0 then
					setPlayerMovementNone()
				end
			else
				setPlayerMovementWalk()
				x,y,z = getPlayerPosition()
				setPlayerPosition(x,y + 40,z+40)
				foundTramwayTicket = 0
			end
		end
	end
end

function lua_draw_2()
	draw3DModel(model.tramway)
end

function lua_unload_2()
	unload3DModel(model.tramway)
end

--============================================================

--=========================================
-- tramway ticket

function lua_load_3()
	model2 = load3DModel('obj_tramway_ticket/tramway_ticket.obj')
end

function lua_manage_3()
	if getDistanceToPlayer() < 15 then
		foundTramwayTicket = 1
	end
end

function lua_draw_3()
	if foundTramwayTicket == 0 then
		draw3DModel(model2)
	end
end

function lua_unload_3()
	unload3DModel(model2)
end

--============================================================
-- magic sphere

nb_visibles_spheres = 0

function lua_load_4()
	model.magic_sphere = load3DModel('obj_magic_sphere/magic_sphere.obj')
end

function lua_manage_4()
	if getDistanceToPlayer() < 15 and getTime() > last_load_or_save_time + 1000 then
		callSaveGame()
		description = 'Game saved.'
		time_description = 1000 + getTime()
		last_load_or_save_time = getTime()
	end
end

function lua_draw_4()
	nb_visibles_spheres=nb_visibles_spheres+1
	if getTime() > last_load_or_save_time + 1000 then
		draw3DModel(model.magic_sphere)
	end
end

function lua_unload_4()
	unload3DModel(model.magic_sphere)
end

--============================================================
-- character

function lua_load_5()
	model.character = load3DModel('obj_player_magician/player_frame1.obj')
end

function lua_manage_5()
	if getDistanceToPlayer() < 20 then
		text_balloon = 'Hello, how are you my friend?\n\nThere is a "end" cube in this world, \ncan you find it?'
	else
		text_balloon = ''
	end
end

function lua_draw_5()
	rotatePosition(0,90,0)
	draw3DModel(model.character)
end

function lua_unload_5()
	unload3DModel(model.character)
end

--============================================================
-- projectile
projectile_speed={x=0,y=0,z=0}

function lua_load_6()
	model.projectile = load3DModel('obj_magic_sphere/magic_sphere.obj')
end

function lua_manage_6()
	local v1,v2,v3,v4,v5 = getObjectLocalVariables()
	--print (v1..','..v2..','..v3..','..v4..','..v5)
	if v4 == 0 then --initialization
		local angle_x, angle_y, dist = getCameraAngleAndDistance()
		v1, v2, v3 = cartesianCoordFromSphericalCoord(angle_x,angle_y, 1.0)
		v4=1
	end
	local duration = getPeriodDuration()/100
	local x,y,z = getCurrentObjectPosition()
	for i = 0, duration, 1 do
		v2=v2-0.00005
		setCurrentObjectPosition(x+v1,y+v2,z+v3)
	end
	local x,y,z = getCurrentObjectPosition()
	if isCollidingPoint(x,y,z) and v4 == 1 then
		removeCube(x,y,z)
		autoDestroyCurrentObject()
	elseif y < -100 then
		autoDestroyCurrentObject()
	end
	setObjectLocalVariables(v1,v2,v3,v4,v5)
end

function lua_draw_6()
	draw3DModel(model.projectile)
end

function lua_unload_6()
	unload3DModel(model.projectile)
end

--============================================================
-- load and save

function callSaveGame()
	setGlobalVar('foundTramwayTicket', foundTramwayTicket)
	saveGame()
end

function callLoadGame()
	loadGame();
	is_game_over = 0
	last_load_or_save_time = getTime()
	foundTramwayTicket = getGlobalVar('foundTramwayTicket')
end

--============================================================
-- main function

function lua_loop()
	
	clearBackground()
	if is_night then
		setFogColor(0,0,0)
	else
		setFogColor(180,200,255)
	end
	--setFogColor(100,128,255)
		
	--==============================
	-- play the music
	
	if getTime() > time_main_music then
		time_main_music = getTime() + 500000
		playSound(sound.main_music)
	end
	if getTime() > time_quiet_music then
		time_quiet_music = getTime() + 500000
		playSound(sound.quiet_music)
	end
	
	--==============================
	-- draw 3d world

	time1 = draw3DWorld()
	
	if isEditorMode() == 0 then
		
		x, y, z = getPlayerPosition()
		
		-- fall into the water
		if y < -15.0 and not(getPlayerMovementType() == 2) then 
			setPlayerMovementFly()
		end
		if y < -0.0 and not is_player_in_water then
			playSound(sound.splash)
			is_player_in_water = true
		end
		
		-- come from the water
		if y > 0.0 and not(getPlayerMovementType() == 0) then 
			setPlayerMovementWalk()
		end
		if y > 0.0 and is_player_in_water then
			playSound(sound.splash)
			is_player_in_water = false
		end
		
		if getKey('l') == 1 then
			callLoadGame()	
		end
		
		-- flying
		if getKey('r')  == 1 then
			speedx, speedy, speedz = getPlayerSpeed()
			setPlayerSpeed(speedx, 1.0, speedz)
		end
		
		-- delta_wing
		local left_click, middle_click, right_click, mouse_pos_x, mouse_pos_y = getMouseInfos()
		if left_click  == 1 then
			speedx, speedy, speedz = getPlayerSpeed()
			if speedy < -0.0 then
				setCurrentPosition(getPlayerPosition())
				local angle_x, angle_y, dist = getCameraAngleAndDistance()
				rotatePosition(0,angle_y,0)
				draw3DModel(model.delta_wing)
				if speedx > 0.0 then
					setPlayerSpeed(6.0, -0.2, speedz)
				else
					setPlayerSpeed(speedx, -0.2, speedz)
				end
			end
		end

		if getKey('b') == 1 and is_night == true then
			is_night = false
			changeConfigValueString("skybox_directory", "skybox_day")
			changeConfigValueString("player_model_directory", "obj_player_magician")
		end
		if getKey('n') == 1 and is_night == false then
			is_night = true
			changeConfigValueString("skybox_directory", "skybox_night")
			changeConfigValueString("player_model_directory", "obj_player_robot")
		end
		
		if getKey('y') == 1 then
			x, y, z = getPlayerPosition()
			startCubeCreation(x,y,z)
		end
		
		local is_finished, x1,y1,z1,x2,y2,z2 = manageCubeCreation()
		
		if is_finished == 1 then
			createCube(x1,y1,z1,x2,y2,z2,2)
			is_finished = 0
		end
		
		if getKey('u') == 0 then
			no_key_shoot = 1
		end
		
		if no_key_shoot ==1 and getKey('u') == 1 then
			local x,y,z=getPlayerPosition()
			createObject(x,y+5,z,6)
			no_key_shoot = 0
			--setPlayerMovementFly()
		end
		--if getKey('u') == 1 then
		--	angle_x, angle_y, distance = getCameraAngleAndDistance()
		--	setCameraAngleAndDistance(angle_x, angle_y, distance+1)
		--end
	end
	
	--lua standard libraries test
	--print (math.cos(0.5).."-")
	
	--==============================
	-- draw 2d
	
	time2 = draw2DWorld()
	
	nb_visibles_spheres=0
	
	-- character text
	if text_balloon ~= '' then
		draw2DSprite(balloon, 10,10)
		drawText(text_balloon,40, 40)
	end
	
	-- contextual text
	if description ~= ''  then
		drawText(description, 10, 10)
		if getTime() > time_description then
			description = ''
		end
	end
	
	if is_player_in_water then
		setPlayerMovementNone()
		setColor(255,255,255,255)
		local win_width, win_height = getWindowSize()
		draw2DSprite(fall_in_water_sprite, win_width/2 - 128, win_height/2 - 128)
		local px,py,pz = getPlayerPosition()
		setPlayerPosition(px, -10.0, pz)
		--drawTextCenter('Don\'t fall into the water ...')
		--drawText('Press Esc to quit, or "L" to \nreload from last saved sphere', width/2 - 200, height/2+50)
	end

	-- gameover display
	if is_game_over == 1 then
		setPlayerMovementNone()
		drawSpriteFullscreen(black_sprite)
		setColor(255,255,255,255)
		drawTextCenter('Game over')
		local width, height = getWindowSize()
		drawText('Press Esc to quit, or "L" to \nreload from last saved sphere', width/2 - 200, height/2+50)
	end	
	
	drawText('u: shoot spheres\nr: fly\nn/b: night robot / day magician\ny: create cube', 10, 10)

end

--============================================================
